@@ -1,4 +1,7 @@
-from .graph import (NodeGraph, EVT_GSNODEGRAPH_NODESELECT, 
+from .graph import (NodeGraph,
+                    ID_CONTEXTMENU_DUPLICATENODE,
+                    ID_CONTEXTMENU_DELETENODES,
+                    EVT_GSNODEGRAPH_NODESELECT, 
                     EVT_GSNODEGRAPH_NODEDISCONNECT,
                     EVT_GSNODEGRAPH_NODECONNECT,
                     EVT_GSNODEGRAPH_MOUSEZOOM)
