@@ -20,7 +20,9 @@ import wx.lib.agw.flatmenu as flatmenu
 from wx.lib.newevent import NewCommandEvent
 
 from gsnodegraph.node import NodeWire
-from gsnodegraph.constants import *
+from gsnodegraph.constants import (GRAPH_BACKGROUND_COLOR, SOCKET_OUTPUT,
+                                   SELECTION_BOX_COLOR, SELECTION_BOX_BORDER_COLOR,
+                                   DEFAULT_WIRE_CURVATURE)
 
 from .utils.z_matrix import ZMatrix
 
@@ -46,6 +48,8 @@ class NodeGraph(wx.ScrolledCanvas):
         self.matrix.Reset()
         self.identity.Reset()
         self.previous_position = None
+
+        self._backgroundimage = None
         self._buffer = None
         self._zoom = 100
 
@@ -67,6 +71,8 @@ class NodeGraph(wx.ScrolledCanvas):
         self._bbox_rect = None
         self._bbox_start = None
 
+        self._wire_curvature = DEFAULT_WIRE_CURVATURE
+
         kwds["style"] = kwds.get("style", 0) | wx.DEFAULT_FRAME_STYLE
         wx.ScrolledCanvas.__init__(self, parent, *args, **kwds)
 
@@ -115,7 +121,7 @@ class NodeGraph(wx.ScrolledCanvas):
     def OnSize(self, event):
         Size = self.ClientSize
         self._buffer = wx.Bitmap(*Size)
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def OnLeftDown(self, event):
         pnt = event.GetPosition()
@@ -137,14 +143,9 @@ class NodeGraph(wx.ScrolledCanvas):
                 if self._src_socket._direction == SOCKET_OUTPUT:
                     pnt1 = self._src_node.pos + self._src_socket.pos
 
-                    self._tmp_wire = NodeWire(
-                        self,
-                        pnt1,
-                        winpnt,
-                        None,
-                        None,
-                        self._src_socket._direction
-                    )
+                    self._tmp_wire = NodeWire(self, pnt1, winpnt, None, None,
+                                              self._src_socket._direction,
+                                              self._wire_curvature)
 
                 # If this is an input socket, we disconnect any already-existing
                 # sockets and connect the new wire. We do not allow disconnections
@@ -158,7 +159,7 @@ class NodeGraph(wx.ScrolledCanvas):
                             self.DisconnectNodes(self._src_socket, dst)
 
                     # Refresh the nodegraph
-                    self.UpdateDrawing()
+                    self.UpdateNodeGraph()
 
                     # Don't allow a wire to be pulled out from an input node
                     if self._src_socket._direction == SOCKET_OUTPUT:
@@ -169,14 +170,9 @@ class NodeGraph(wx.ScrolledCanvas):
                         pnt1 = self._src_socket._node._pos + self._src_socket._pos
 
                         # Draw the temp wire with the new values
-                        self._tmp_wire = NodeWire(
-                            self,
-                            pnt1,
-                            winpnt,
-                            None,
-                            None,
-                            self._src_socket._direction,
-                        )
+                        self._tmp_wire = NodeWire(self, pnt1, winpnt, None, None,
+                                                  self._src_socket._direction,
+                                                  self._wire_curvature)
 
                         # Important: we re-assign the source node variable
                         self._src_node = self._src_socket._node
@@ -191,7 +187,7 @@ class NodeGraph(wx.ScrolledCanvas):
         self._last_pnt = winpnt
 
         # Refresh the nodegraph
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def OnLeftUp(self, event):
         pnt = event.GetPosition()
@@ -213,9 +209,9 @@ class NodeGraph(wx.ScrolledCanvas):
                 # Make sure not to allow different datatypes or
                 # the same 'socket type' to be connected!
                 if dst_socket is not None:
-                    if self._src_socket._direction != dst_socket._direction \
-                        and self._src_socket._datatype == dst_socket._datatype \
-                        and self._src_node != dst_node:
+                    if (self._src_socket._direction != dst_socket._direction
+                        and self._src_socket._datatype == dst_socket._datatype
+                        and self._src_node != dst_node):
 
                         # Only allow a single wire to be connected to any one input.
                         if self.SocketHasWire(dst_socket) is not True:
@@ -245,7 +241,7 @@ class NodeGraph(wx.ScrolledCanvas):
         self.SendNodeSelectEvent()
 
         # Refresh the nodegraph
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def OnMotion(self, event):
         pnt = event.GetPosition()
@@ -256,7 +252,7 @@ class NodeGraph(wx.ScrolledCanvas):
             if self._src_node is None and self._bbox_start != None:
                 rect = wx.Rect(topLeft=self._bbox_start, bottomRight=winpnt)
                 self._bbox_rect = rect
-                self.UpdateDrawing()
+                self.UpdateNodeGraph()
 
         # If the MMB is down, calculate the scrolling of the graph
         if event.MiddleIsDown() is True and event.Dragging():
@@ -264,7 +260,7 @@ class NodeGraph(wx.ScrolledCanvas):
             dy  =(winpnt[1] - self._middle_pnt[1])
             self.ScrollNodeGraph(dx, dy)
             self.ScenePostPan(dx, dy)
-            self.UpdateDrawing()
+            self.UpdateNodeGraph()
 
         if event.LeftIsDown() and self._src_node != None and event.Dragging():
             if self._src_socket is None:
@@ -294,7 +290,7 @@ class NodeGraph(wx.ScrolledCanvas):
                 if winpnt != None:
                     self._tmp_wire.pnt2 = winpnt
 
-            self.UpdateDrawing()
+            self.UpdateNodeGraph()
 
     def OnDeleteNodes(self, event):
         self.DeleteNodes()
@@ -309,15 +305,15 @@ class NodeGraph(wx.ScrolledCanvas):
         # nodes' properties are not still shown!
         self.SendNodeSelectEvent()
 
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def OnMuteNode(self, event):
         self._active_node.SetMuted(True)
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def OnUnmuteNode(self, event):
         self._active_node.SetMuted(False)
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def OnSelectAllNodes(self, event):
         """ Event that selects all the nodes in the Node Graph. """
@@ -327,12 +323,12 @@ class NodeGraph(wx.ScrolledCanvas):
                 node.SetActive(False)
             node.SetSelected(True)
             self._selected_nodes.append(node)
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def OnDeselectAllNodes(self, event):
         """ Event that deselects all the currently selected nodes. """
         self.DeselectNodes()
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def OnDuplicateNode(self, event):
         """ Event that duplicates the currently selected node. """
@@ -401,10 +397,10 @@ class NodeGraph(wx.ScrolledCanvas):
                                                           wx.ITEM_NORMAL)
         self.context_menu.AppendItem(deselectallnodes_menuitem)
 
-
     def DrawSelectionBox(self, dc, rect):
-        dc.SetPen(wx.Pen(wx.Colour('#C2C2C2'), 2.5, wx.PENSTYLE_SHORT_DASH))
-        dc.SetBrush(wx.Brush(wx.Colour(100, 100, 100, 56), wx.SOLID))
+        dc.SetPen(wx.Pen(wx.Colour(SELECTION_BOX_BORDER_COLOR), 2,
+                  wx.PENSTYLE_SHORT_DASH))
+        dc.SetBrush(wx.Brush(wx.Colour(SELECTION_BOX_COLOR)))
         dc.DrawRectangle(rect)
 
     def SetZoomLevel(self, zoom, x=0, y=0):
@@ -413,20 +409,21 @@ class NodeGraph(wx.ScrolledCanvas):
             y = self.Size[1]/2
         self.ScenePostScale(zoom, zoom, x, y)
         self.UpdateZoomValue()
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def OnMousewheel(self, event):
         rotation = event.GetWheelRotation()
         mouse = event.GetPosition()
-        if rotation > 1:
+
+        if rotation > 1 and self._zoom < 310:
             self.ScenePostScale(1.1, 1.1, mouse[0], mouse[1])
 
-        elif rotation < -1:
+        elif rotation < -1 and self._zoom > 60:
             self.ScenePostScale(0.9, 0.9, mouse[0], mouse[1])
 
         self.UpdateZoomValue()
         self.SendMouseZoomEvent()
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def OnMiddleDown(self, event):
         """ Event that updates the mouse cursor. """
@@ -481,7 +478,7 @@ class NodeGraph(wx.ScrolledCanvas):
         rgn.Offset(x, y)
         return rgn.GetBox()
 
-    def UpdateDrawing(self):
+    def UpdateNodeGraph(self):
         dc = wx.MemoryDC()
         dc.SelectObject(self._buffer)
         dc = wx.GCDC(dc)
@@ -495,27 +492,49 @@ class NodeGraph(wx.ScrolledCanvas):
         self.Update()
 
     def OnDrawBackground(self, dc):
-        dc.SetBackground(wx.Brush('#2E2E2E'))
+        dc.SetBackground(wx.Brush(wx.Colour(GRAPH_BACKGROUND_COLOR)))
         dc.Clear()
-        dc.SetBrush(wx.Brush(wx.Colour('#282828'), wx.CROSS_HATCH))
-        dc.DrawRectangle(0, 0, self.Size[0], self.Size[1])
 
     def OnDrawScene(self, dc):
+        # Draw background
+        if self._backgroundimage != None:
+            image = self._backgroundimage
+
+            x = (self.GetSize()[0]/2.0 - image.Width/2.0)
+            y = (self.GetSize()[1]/2.0 - image.Height/2.0)
+            pnt = self.ConvertCoords(wx.Point(x, y))
+            dc.DrawBitmap(image, pnt[0], pnt[1], useMask=False)
+
+        # Draw nodes
         for node in self._nodes:
             self._nodes[node].Draw(dc)
 
+        # Draw temporary wires
         if self._tmp_wire != None:
             self._tmp_wire.Draw(dc)
 
+        # Draw wires
         for wire in self._wires:
             wire.Draw(dc)
 
+        # Draw selection box
         if self._bbox_start != None and self._bbox_rect != None:
             self.DrawSelectionBox(dc, self._bbox_rect)
 
     def OnDrawInterface(self, dc):
         pass
 
+    def SetNodeWireCurvature(self, curvature):
+        self._wire_curvature = curvature
+
+        # Change existing wires
+        for wire in self._wires:
+            wire.SetCurvature(curvature)
+        #self.UpdateNodeGraph()
+
+    def SetBackgroundImage(self, image):
+        self._backgroundimage = image
+
     def HandleNodeSelection(self):
         # Set the active node
         if self._active_node is None:
@@ -533,8 +552,7 @@ class NodeGraph(wx.ScrolledCanvas):
         # When a node is active, all the selected nodes
         # need to be set to the unselected state.
         if self._selected_nodes != []:
-            for node in self._selected_nodes:
-                node.SetSelected(False)
+            [node.SetSelected(False) for node in self._selected_nodes]
 
     def BoxSelectHitTest(self, bboxrect):
         """ Hit-test for box selection. """
@@ -551,8 +569,7 @@ class NodeGraph(wx.ScrolledCanvas):
 
     def DeselectNodes(self):
         """ Deselect everything that is selected or active. """
-        for node in self._selected_nodes:
-            node.SetSelected(False)
+        [node.SetSelected(False) for node in self._selected_nodes]
 
         self._selected_nodes = []
 
@@ -566,12 +583,12 @@ class NodeGraph(wx.ScrolledCanvas):
         for node in self._nodes:
             # Inflate the rect so that the node sockets are
             # highly sensitive to clicks for easier connections.
-            node_rect = self._nodes[node].GetRect().Inflate(6, 6)
+            node_rect = self._nodes[node].GetRect().Inflate(7, 7)
             if mouse_rect.Intersects(node_rect):
                 return self._nodes[node]
 
             # Refresh the nodegraph
-            self.UpdateDrawing()
+            self.UpdateNodeGraph()
 
     def DeleteNodes(self):
         """ Delete the currently selected nodes. This will refuse
@@ -587,7 +604,7 @@ class NodeGraph(wx.ScrolledCanvas):
         self._selected_nodes = []
 
         if (self._active_node != None and
-           self._active_node.IsOutputNode() != True):
+            self._active_node.IsOutputNode() != True):
             self.DeleteNode(self._active_node)
             self._active_node = None
 
@@ -595,7 +612,7 @@ class NodeGraph(wx.ScrolledCanvas):
         # nodes' properties are not still shown!
         self.SendNodeSelectEvent()
 
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def DuplicateNode(self, node):
         """ Duplicates the given ``Node`` object with its properties.
@@ -603,14 +620,11 @@ class NodeGraph(wx.ScrolledCanvas):
         :returns: the duplicate ``Node`` object
         """
         if node.IsOutputNode() is not True:
-            duplicate_node = self.AddNode(
-                node.GetIdname(),
-                location="CURSOR"
-            )
+            duplicate_node = self.AddNode(node.GetIdname(), location="CURSOR")
 
             # TODO: Assign the same properties to the duplicate node object
 
-            self.UpdateDrawing()
+            self.UpdateNodeGraph()
             return duplicate_node
 
     def AddNode(self, idname, pos=(0, 0), location="POSITION"):
@@ -634,7 +648,8 @@ class NodeGraph(wx.ScrolledCanvas):
         pt2 = dst_socket._node._pos + dst_socket._pos
         _dir = src_socket._direction
 
-        wire = NodeWire(src_socket, pt1, pt2, src_socket, dst_socket, _dir)
+        wire = NodeWire(src_socket, pt1, pt2, src_socket, dst_socket,
+                        _dir, self._wire_curvature)
         wire.srcnode = src_socket._node
         wire.dstnode = dst_socket._node
         wire._srcsocket = src_socket
@@ -663,7 +678,7 @@ class NodeGraph(wx.ScrolledCanvas):
                 # connected to this node.
                 self.DisconnectNodes(wire.srcsocket, wire.dstsocket)
         del self._nodes[node._id]
-        self.UpdateDrawing()
+        self.UpdateNodeGraph()
 
     def SendNodeSelectEvent(self):
         wx.PostEvent(self,
