@@ -17,8 +17,14 @@
 import wx
 import uuid
 
+from gsnodegraph.assets.bitmaps import ICON_IMAGE
+
 from .socket import NodeSocket
-from ..constants import *
+from ..constants import (NODE_DEFAULT_WIDTH, NODE_DEFAULT_HEIGHT,
+                         NODE_HEADER_MUTED_COLOR, NODE_HEADER_CATEGORY_COLORS,
+                         SOCKET_INPUT, SOCKET_OUTPUT, NODE_THUMB_PADDING, NODE_Y_PADDING,
+                         NODE_NORMAL_COLOR, NODE_MUTED_COLOR, NODE_THUMB_BORDER_COLOR,
+                         NODE_BORDER_NORMAL_COLOR, NODE_BORDER_SELECTED_COLOR)
 from ..assets import (ICON_BRUSH_CHECKERBOARD, ICON_IMAGE)
 
 
@@ -29,6 +35,7 @@ class NodeBase(object):
         self._idname = None
         self._pos = wx.Point(0, 0)
         self._size = wx.Size(NODE_DEFAULT_WIDTH, NODE_DEFAULT_HEIGHT)
+
         self._expanded = False
         self._selected = False
         self._active = False
@@ -40,14 +47,24 @@ class NodeBase(object):
         self._isoutput = False
         self._label = ""
         self._category = "DEFAULT"
-        self._headercolor = "#fff"
+        self._headercolor = wx.Colour(NODE_HEADER_CATEGORY_COLORS["DEFAULT"])
 
-    def _Init(self, idname):
+        self._thumbnail = self._CreateEmptyBitmap()
+        self._expandicon_bmp = ICON_IMAGE.GetBitmap()
+        self._checkerboard_bmp = ICON_BRUSH_CHECKERBOARD.GetBitmap()
+
+    def _Init(self, idname) -> None:
         self.InitSockets()
         self.InitHeaderColor()
         self.InitSize()
         self.SetIdName(idname)
 
+    def _CreateEmptyBitmap(self) -> wx.Bitmap:
+        img = wx.Image(120, 120)
+        img.SetMaskColour(0,0,0)
+        img.InitAlpha()
+        return img.ConvertToBitmap()
+
     @property
     def nodegraph(self):
         return self._nodegraph
@@ -104,18 +121,19 @@ class NodeBase(object):
     def expanded(self, expanded: bool) -> None:
         self._expanded = expanded
 
-    def NodeOutputDatatype(self):
+    def NodeOutputDatatype(self) -> str:
         return "RGBAIMAGE"
 
     def AddSocket(self, label, color, direction) -> None:
         self.ArrangeSockets()
 
-    def HitTest(self, pos: wx.Point):
+    def HitTest(self, pos: wx.Point) -> None:
         # Handle expanding the node to show thumbnail hittest
-        icon_rect = self._expandicon_rect.Inflate(8, 8)
-        mouse_rect = wx.Rect(pos[0], pos[1], 2, 2)
-        if mouse_rect.Intersects(icon_rect) and wx.GetMouseState().LeftIsDown():
-            self.ToggleExpand()
+        if self.HasThumbnail():
+            icon_rect = self._expandicon_rect.Inflate(8, 8)
+            mouse_rect = wx.Rect(pos[0], pos[1], 2, 2)
+            if mouse_rect.Intersects(icon_rect) and wx.GetMouseState().LeftIsDown():
+                self.ToggleExpand()
 
         # Handle socket hittest
         for socket in self._sockets:
@@ -126,7 +144,7 @@ class NodeBase(object):
         pass
 
     def InitHeaderColor(self) -> None:
-        self._headercolor = NODE_CATEGORY_COLORS[self.GetCategory()]
+        self._headercolor = wx.Colour(NODE_HEADER_CATEGORY_COLORS[self.GetCategory()])
 
     def InitSockets(self) -> None:
         sockets = []
@@ -140,7 +158,15 @@ class NodeBase(object):
             ins.append((param.label, param.idname, param.datatype))
 
         if self.IsOutputNode() is not True:
-            outs = [('Output', 'Output', self.NodeOutputDatatype())]
+            # TODO: eventually, we want to have the ability to edit the label
+            data_type = self.NodeOutputDatatype()
+            if data_type == "RGBAIMAGE":
+                idname = "output_image"
+                label = "Image"
+            else:
+                idname = "output_value"
+                label = "Value"
+            outs = [(label, idname, data_type)]
 
         x, y, w, h = self.GetRect()
         x, y = self.pos
@@ -168,8 +194,9 @@ class NodeBase(object):
         # Calculate the normal size of the node to fit
         # the amount of sockets the node has. The expanded size
         # is calculated to be the normal size plus the image thumbnail size.
-        self._expandedsize = wx.Size(NODE_DEFAULT_WIDTH,
-                                     self._lastsocketpos+NODE_DEFAULT_WIDTH+(NODE_Y_PADDING*2))
+        calc_height = self._lastsocketpos + self._thumbnail.Height + NODE_THUMB_PADDING * 2
+        self._expandedsize = wx.Size(NODE_DEFAULT_WIDTH, calc_height)
+
         self._normalsize = wx.Size(NODE_DEFAULT_WIDTH,
                                    self._lastsocketpos+(NODE_Y_PADDING*2))
 
@@ -179,6 +206,12 @@ class NodeBase(object):
         else:
             self.SetSize(self._normalsize)
 
+    def HasThumbnail(self) -> bool:
+        if self.NodeOutputDatatype() == "RGBAIMAGE":
+            return True
+        else:
+            return False
+
     def IsOutputNode(self) -> bool:
         """ Override method to set whether the node is the output or not. """
         return self._isoutput
@@ -229,6 +262,8 @@ class NodeBase(object):
 
     def SetMuted(self, muted=True) -> None:
         self.muted = muted
+        self.SetExpanded(False)
+        self.SetSize(self._normalsize)
 
     def IsExpanded(self) -> bool:
         return self.expanded
@@ -237,41 +272,58 @@ class NodeBase(object):
         self.expanded = expanded
 
     def ToggleExpand(self) -> None:
-        if self.IsExpanded() is True:
-            self.SetExpanded(False)
-            self.SetSize(self._normalsize)
-        elif self.IsExpanded() is False:
-            self.SetExpanded(True)
-            self.SetSize(self._expandedsize)
+        if self.HasThumbnail():
+            if self.IsExpanded() is True:
+                self.SetExpanded(False)
+                self.SetSize(self._normalsize)
+            elif self.IsExpanded() is False:
+                self.SetExpanded(True)
+                self.SetSize(self._expandedsize)
 
     def GetSockets(self) -> list:
         return self._sockets
 
-    def Draw(self, dc):
+    def SetThumbnail(self, thumb) -> None:
+        if self.HasThumbnail():
+            self._thumbnail = thumb
+            self.UpdateExpandSize()
+
+    def UpdateExpandSize(self) -> None:
+        calc_height = self._lastsocketpos + self._thumbnail.Height + NODE_THUMB_PADDING * 2
+        self._expandedsize = wx.Size(NODE_DEFAULT_WIDTH, calc_height)
+        self.SetSize(self._expandedsize)
+
+    def Draw(self, dc) -> None:
         x, y = self.GetPosition()
         w, h = self.GetSize()
 
         # Node body and border
         if self.IsSelected() or self.IsActive():
-            dc.SetPen(wx.Pen(wx.Colour(255, 255, 255, 255), 1))
+            border_color = NODE_BORDER_SELECTED_COLOR
         else:
-            dc.SetPen(wx.Pen(wx.Colour(31, 31, 31, 255), 1))
+            border_color = NODE_BORDER_NORMAL_COLOR
         if self.IsMuted():
-            color = wx.Colour(70, 70, 70, 90)
+            node_color = NODE_MUTED_COLOR
         else:
-            color = wx.Colour(70, 70, 70, 255)
-        dc.SetBrush(wx.Brush(color))
+            node_color = NODE_NORMAL_COLOR
+        dc.SetPen(wx.Pen(wx.Colour(border_color), 1))
+        dc.SetBrush(wx.Brush(wx.Colour(node_color)))
         dc.DrawRoundedRectangle(x, y, w, h, 3)
 
         # Node header
         dc.SetPen(wx.Pen(wx.TRANSPARENT_PEN))
         if self.IsMuted():
-            color = wx.Colour(70, 70, 70, 255)
+            header_color = wx.Colour(NODE_HEADER_MUTED_COLOR)
+            bottom_color = wx.Colour(NODE_HEADER_MUTED_COLOR).ChangeLightness(80)
         else:
-            color = wx.Colour(self._headercolor).ChangeLightness(80)
-        dc.SetBrush(wx.Brush(color))
-        dc.DrawRoundedRectangle(x+1, y+1, w-2, 12, 2)
-        dc.DrawRectangle(x+1, y+10, w-2, 12)
+            header_color = wx.Colour(self._headercolor).ChangeLightness(70)
+            bottom_color = wx.Colour(self._headercolor).ChangeLightness(55)
+        dc.SetBrush(wx.Brush(header_color))
+        dc.DrawRoundedRectangle(x+1, y+1, w-2, 24, 3)
+
+        # Bottom border of the node header (to cover up the rounded bottom)
+        dc.SetBrush(wx.Brush(bottom_color))
+        dc.DrawRectangle(x+1, y+23, w-2, 2)
 
         # Node name label
         if self.IsMuted():
@@ -279,27 +331,29 @@ class NodeBase(object):
         else:
             color = wx.Colour('#fff').ChangeLightness(85)
         dc.SetTextForeground(color)
-        dc.DrawText(self.GetLabel(), x+10, y)
-
-        # Expand node thumbnail icon
-        self._expandicon_rect = wx.Rect(x+NODE_DEFAULT_WIDTH-24, y+3, 16, 16)
-        dc.DrawBitmap(ICON_IMAGE.GetBitmap(),
-                      self._expandicon_rect[0],
-                      self._expandicon_rect[1],
-                      True)
+        dc.DrawText(self.GetLabel(), x+10, y+1)
 
         # Node sockets
-        for socket in self._sockets:
-            socket.Draw(dc)
+        [socket.Draw(dc) for socket in self._sockets]
+
+        # Expand node thumbnail icon
+        if self.HasThumbnail() == True and self.IsMuted() != True:
+            self._expandicon_rect = wx.Rect(x+NODE_DEFAULT_WIDTH-28, y+4, 16, 16)
+            dc.DrawBitmap(self._expandicon_bmp, self._expandicon_rect[0],
+                        self._expandicon_rect[1], True)
 
         # Thumbnail
-        if self.IsExpanded():
+        if self.IsExpanded() and self.HasThumbnail():
             # Calculate the coords for the placement of the thumbnail
             thumb_rect = wx.Rect((x+NODE_THUMB_PADDING/2),
                                   y+self._lastsocketpos+(NODE_Y_PADDING*2),
-                                  NODE_DEFAULT_WIDTH-NODE_THUMB_PADDING, NODE_DEFAULT_WIDTH-NODE_THUMB_PADDING)
+                                  NODE_DEFAULT_WIDTH-NODE_THUMB_PADDING,
+                                  self._thumbnail.Height)
 
             # Draw thumbnail border and background
-            dc.SetPen(wx.Pen(wx.Colour("#2B2B2B"), 1))
-            dc.SetBrush(wx.Brush(ICON_BRUSH_CHECKERBOARD.GetBitmap()))
+            dc.SetPen(wx.Pen(wx.Colour(NODE_THUMB_BORDER_COLOR), 1))
+            dc.SetBrush(wx.Brush(self._checkerboard_bmp))
             dc.DrawRectangle(thumb_rect)
+
+            # Draw the thumbnail
+            dc.DrawBitmap(self._thumbnail, thumb_rect[0], thumb_rect[1], True)
