@@ -21,18 +21,18 @@ from ..constants import WIRE_NORMAL_COLOR, WIRE_ACTIVE_COLOR
 
 class NodeWire(object):
     """ Wire for showing a connection between two nodes. """
-    def __init__(self, parent, pnt1, pnt2, srcsocket, dstsocket, direction):
+    def __init__(self, parent, pnt1, pnt2, srcsocket, dstsocket, direction, curvature):
         self._parent = parent
         self._pnt1 = pnt1
         self._pnt2 = pnt2
         self._srcsocket = srcsocket
         self._dstsocket = dstsocket
+        self._curvature = curvature
+        self._direction = direction
+
         self._srcnode = None
         self._dstnode = None
-
         self._active = False
-        self._curvature = 8
-        self._direction = direction
 
     @property
     def parent(self):
@@ -98,6 +98,9 @@ class NodeWire(object):
     def direction(self, direction):
         self._direction = direction
 
+    def SetCurvature(self, curvature):
+        self.curvature = curvature
+
     def GetRect(self):
         min_x = min(self.pnt1[0], self.pnt2[0])
         min_y = min(self.pnt1[1], self.pnt2[1])
@@ -123,15 +126,17 @@ class NodeWire(object):
             pnts.append(self.pnt2)
 
             if self.active is True:
-                dc.SetPen(wx.Pen(wx.Colour(WIRE_ACTIVE_COLOR), 3))
+                color = WIRE_ACTIVE_COLOR
             else:
-                dc.SetPen(wx.Pen(wx.Colour(WIRE_NORMAL_COLOR), 3))
+                color = WIRE_NORMAL_COLOR
+            dc.SetPen(wx.Pen(wx.Colour(color), 3))
             dc.DrawSpline(pnts)
 
         else:
             # Otherwise, use a line
             if self.active is True:
-                dc.SetPen(wx.Pen(wx.Colour(WIRE_ACTIVE_COLOR), 3))
+                color = WIRE_ACTIVE_COLOR
             else:
-                dc.SetPen(wx.Pen(wx.Colour(wx.Colour(WIRE_NORMAL_COLOR)), 3))
+                color = WIRE_NORMAL_COLOR
+            dc.SetPen(wx.Pen(wx.Colour(color), 3))
             dc.DrawLine(self.pnt1[0], self.pnt1[1], self.pnt2[0], self.pnt2[1])
